#------------------------------------------------------------------------------
# OIDC Client application deployment/undeployment script
# wsadmin.sh -f setupsaml2.py (saml2IdPUrl) (saml2IdIdentifer) (oidcRpIdentifier) (oidcRpClientName) (oidcRpIntercept) (oidcRpSecret) (oidcRpProviderNo) [name=value ...]
#               Setup and configure this WebSphere server as an OIDC Relying Party.
#               Arguments:
#               oidcIdPUrl - OIDC ID providers URL including the ID, and trailing slash, e.g.,
#                           "https://www.myidp.com:9443/oidc/
#               saml2IdIdentifer - The OIDC ID providers identifier, e.g. "MY_OP"
#               oidcRpIdentifier - The identifier for the relying party, e.g. "libop"
#                                   Used for registering an RP client on an IDP.
#               oidcRpClientName - The client name for the relying party, e.g. "MY_RP"
#               oidcRpIntercept - The intercept filter for the RP, e.g. "/.*"
#               oidcRpSecret - A secret word used for establishing trust between OIDC provider and client.
#               oidcRpProviderNo - a number that allows you to register multiple 
#                            OIDC ID providers on the webserver, e.g. "1"
#               [optional additional name=value configurations for the TAI custom properties.]
#
# wsadmin.sh -f setupsaml2.py setproperty name=value ...
#               Set or add custom properties to the OIDC Relying Party TAI.
#
# wsadmin.sh -f setupsaml2.py.py unconfig
#
# wsadmin.sh -f setupsaml2.py.py addtrustedrealm (trustedrealm)
#               Add a trusted realm to WebSphere. For example if the IDP realm is "MyIDPRealm"
#               you need to add it as a trusted realm for WebSphere to trust it.
#               
#------------------------------------------------------------------------------
import sys

#------------------------------------------------------------------------------
# Print script usage
#------------------------------------------------------------------------------
def printUsage():
	print "Usage: wsadmin -f <WAS_HOME>/bin/setupsaml2.py.py help"
	print "  or:  wsadmin -f <WAS_HOME>/bin/setupsaml2.py.py config (saml2IdPUrl) (saml2IdIdentifer) (oidcRpIdentifier) (oidcRpClientName) (oidcRpIntercept) (oidcRpSecret) (oidcRpProviderNo) [(name)=(value) ... ]"
	print "  or:  wsadmin -f <WAS_HOME>/bin/setupsaml2.py.py setproperty (name)=(value) ... "
	print "  or:  wsadmin -f <WAS_HOME>/bin/setupsaml2.py.py unconfig"
	print ""

#------------------------------------------------------------------------------
# Unconfigure the application
#------------------------------------------------------------------------------
def doUnconfig():
    failOnError = "false"
    print "*** Unconfiguring the OIDC RP interceptor..."
    try:
        AdminTask.unconfigureInterceptor("[-interceptor com.ibm.ws.security.oidc.client.RelyingParty ]")
        # AdminTask.configureTrustAssociation("-enable false")
        AdminConfig.save()
        print "*** Done unconfiguring the OIDC RP interceptor..."
    except:
    	print '!!! Failed to unconfigure the OIDC RP interceptor ', sys.exc_info()
    failOnError = "true"

#------------------------------------------------------------------------------
# Configure a custom property for the OpenIDC RP Trust Association Intercaptor (TAI)
#------------------------------------------------------------------------------
def setCustomPropertyForProvider(providerno, name, value):
    newName = "provider_"+providerno+"." + name;
    nameValue = newName + "=" + value
    setCustomProperty(nameValue)
     
#------------------------------------------------------------------------------
# Configure a custom property for the OpenIDC RP Trust Association Intercaptor (TAI)
# Accepts the namevalue as an "name=value" string
#------------------------------------------------------------------------------
def setCustomProperty(pNameValue): 
    failOnError = "false"
    #"AdminTask.configureInterceptor('[-interceptor com.ibm.ws.security.oidc.client.RelyingParty -customProperties ]') 
    #configArg = '[-interceptor com.ibm.ws.security.oidc.client.RelyingParty -customProperties ["test=blah"]]'
    configArg = '[-interceptor com.ibm.ws.security.oidc.client.RelyingParty -customProperties [\"'+pNameValue+'\"]]'
    print "*** Setting custom propert: " + pNameValue
    try:
    	AdminTask.configureInterceptor(configArg)
    except:
    	print 'Failed to configure the OIDC RP TAI interceptor with custom property ', sys.exc_info()	    
    #AdminTask.configureInterceptor('[-interceptor com.ibm.ws.security.oidc.client.RelyingParty -customProperties [\"'+pNameValue+'\"]]')
    failOnError = "true"
#------------------------------------------------------------------------------
# Configure custom properties from argv for the OpenID RP Trust Association Intercaptor (TAI)
#------------------------------------------------------------------------------
def setCustomPropertiesFromArgv(argv, startAt):
	nowAt=startAt
        stopAt=len(argv)
	while nowAt < stopAt:
		setCustomProperty(argv[nowAt])
		nowAt += 1
	# Save the config after setting all custom properties requested
	AdminConfig.save()
	

#------------------------------------------------------------------------------
# given a URL, get the port number (assumes port 443 by default for https URLS)
#------------------------------------------------------------------------------
def getPortFromUrl(oidcIdPUrl):
	from java.net import URL
	url = URL(oidcIdPUrl)
	port = url.getPort()
	if port==-1 and	url.getProtocol()=="https":
		port=443
	return str(port)

#------------------------------------------------------------------------------
# given a URL, get the host string
#------------------------------------------------------------------------------
def getHostFromUrl(oidcIdPUrl):
	from java.net import URL
	url = URL(oidcIdPUrl)
	host = url.getHost()
	return host


#------------------------------------------------------------------------------
# Initial Configuration for the OpenID RP Trust Association Interceptor (TAI)
#------------------------------------------------------------------------------
def doConfigure(oidcIdPUrl, idpidentifier, rpidentifier, clientid, intercept, secret, providerNo):
	doUnconfig()
	print '*** Enabling trust association'
	AdminTask.configureTrustAssociation("-enable true")
	print '*** Enabled trust association'
	# TODO - don't overwrite, add to the end?
	print '*** Setting InvokeTAIbeforeSSO'
	AdminTask.setAdminActiveSecuritySettings('-customProperties  \'"com.ibm.websphere.security.InvokeTAIbeforeSSO=com.ibm.ws.security.oidc.client.RelyingParty"\'')
	print '*** Done setting InvokeTAIbeforeSSO'
	
	oidcIdPUrlBase = oidcIdPUrl + "endpoint/" + idpidentifier + "/"
	
	print '*** Setting Custom properies for OIDC RP TAI'
	setCustomPropertyForProvider(providerNo, "interceptedPathFilter", intercept)
	setCustomPropertyForProvider(providerNo, "identifier", rpidentifier)
	setCustomPropertyForProvider(providerNo, "clientId", clientid)
	setCustomPropertyForProvider(providerNo, "clientSecret", secret)
	setCustomPropertyForProvider(providerNo, "authorizeEndpointUrl", oidcIdPUrlBase + "authorize")
	setCustomPropertyForProvider(providerNo, "tokenEndpointUrl", oidcIdPUrlBase + "token")
	setCustomPropertyForProvider(providerNo, "introspectEndpoint", oidcIdPUrlBase + "introspect")
	setCustomPropertyForProvider(providerNo, "scope", "openid general")
	setCustomPropertyForProvider(providerNo, "signatureAlgorithm", "HS256")
	AdminConfig.save()

def addTrustedRealm(trustedRealm):
    AdminTask.addTrustedRealms('[-communicationType inbound -realmList %s]' % trustedRealm)
    AdminConfig.save();

#------------------------------------------------------------------------------
# runscript - run the script as a subroutine
#------------------------------------------------------------------------------
def runscript(args):
  failOnError = "true"

  mode = args[0]
  if mode == "help":
  	printUsage()
  else:
    if mode == "config":
        if len(args) > 7:
            argNo = 1
            oidcIdPUrl = args[argNo]
            argNo += 1
            idpidentifer = args[argNo]
            argNo += 1
            rpidentifer = args[argNo]
            argNo += 1
            clientid = args[argNo]
            argNo += 1
            intercept = args[argNo]
            argNo += 1
            secret = args[argNo]
            argNo += 1
            providerNo = args[argNo]
            argNo += 1
            doConfigure(oidcIdPUrl, idpidentifer, rpidentifer, clientid, intercept, secret, providerNo)
            setCustomPropertiesFromArgv(args,argNo)
        else:
            sys.stderr.write("!!! Invalid number of arguments for config operation\n")
            printUsage()
            sys.exit(101)
    elif mode == "setproperty":
        if len(args) > 1 :
            setCustomPropertiesFromArgv(args,1)
        else:
            sys.stderr.write("!!! Invalid number of arguments for setproperty operation\n")
            printUsage()
            sys.exit(101)
    elif mode == "unconfig":
        if len(args) == 1:
            doUnconfig()
        else:
            sys.stderr.write("!!! Invalid number of arguments for unconfig operation\n")
            printUsage()
            sys.exit(101)
    elif mode == "addtrustedrealm":
        if len(args) == 2:
            addTrustedRealm(args[1]);
        else:
            sys.stderr.write("!!! Invalid number of arguments for addtrustedrealm operation\n")
            printUsage()
            sys.exit(101)        
    else:
        sys.stderr.write("Invalid operation\n")
        printUsage()
        sys.exit(101)
  
#------------------------------------------------------------------------------
# Main entry point
#------------------------------------------------------------------------------
if len(sys.argv) > 0:
	runscript(sys.argv)
else:
   	printUsage()


